#include <string.h>
#include "cdf.h"

#include "stack-c.h"
/*
 * Author 
 * S. Steer , Copyright INRIA, Marsh 2006
 */



typedef struct hypermat {
  int type;           /* type of the (elements of the) hyper matrix */
  int it;             /* sub type (in case of type=1 or type=8) */
  int dimsize;        /* number of dimensions of the hyper matrix */
  int size;           /* total number of elements : size = dims[0]x... x dims[dimsize-1] */
  int *dims;          /* number of elements in each dimension */
  double *R, *I;      /* in case of type=1 points to the elements (I being used if it=1)  */
  void *P;            /* in case of type=4 or 8 points to the elements */
} HyperMat;

#define I_CHAR 1
#define I_INT8 1
#define I_UINT8 11
#define I_INT16 2 
#define I_INT32 4
#define I_UCHAR 11
#define I_UINT16 12 
#define I_UINT32 14

/* Function prototypes */
extern int C2F(createlistcvarfrom)(integer *lnumber, integer *number, char *typex, integer *it, integer *m, integer *n, integer *lr, integer *lc, integer *lar, integer *lac, unsigned long type_len);
extern int C2F(createlistvarfrom) (integer *lnumber, integer *number, char *type__, integer *m, integer *n, integer *lr, integer *lar, unsigned long type_len);
static int cre_hmat(int pos, HyperMat *H);
extern int createstringvect(integer *lw, integer *m, integer *nchar, integer *lr);
#define CreateHMat(pos,H) if (! cre_hmat(pos,H)) { return 0;}
#define CreateStringVector(n,m,nchar,lr) if(!createstringvect((c_local=n,&c_local),m,nchar,lr)) { return 0;  }



int CreateCDFVar(int num, long CDFDataType,int SciNdims,int *dims,void **data,int *element_size)
{
  HyperMat H;
  int i,ntot,l,n1;

  ntot=1;for (i=0;i<SciNdims;i++) ntot*=dims[i];
  /*  sciprint("CDFDataType:%d, SciNdims:%d, ntot:%d, dims:[",CDFDataType, SciNdims,ntot);
      for (i=0;i<SciNdims;i++)  sciprint("%d ",dims[i]);
      sciprint("]\n");*/
  switch (CDFDataType) {

  case CDF_EPOCH: case CDF_REAL8: case CDF_DOUBLE:
    /* 8 byte floating point */
    //if (sizeof(double)!=8) mexErrMsgTxt("cdfread: unexpected DOUBLE size when converting CDF_EPOCH");
    if (SciNdims==2) {
      CreateVar(num,"d",  dims, dims+1,&l);
      *data=(void *)stk(l);
    }
    else {
      H.dimsize = SciNdims;
      H.size = ntot;
      H.it = 0;
      H.type = 1;
      CreateHMat(num,&H);
      for (i=0;i<SciNdims;i++)  H.dims[i]=dims[i];
      *data=(void *)H.R;
    }
    *element_size=8;
    break;
  case CDF_REAL4: case CDF_FLOAT:
    /* 4 byte floating point */
    //if (sizeof(float)!=4) mexErrMsgTxt("cdfread: unexpected FLOAT size when converting CDF_FLOAT");
    if (SciNdims==2) {
      CreateVar(num,"d",  dims, dims+1,&l);
      *data=(void *)stk(l);
    }
    else {
      H.dimsize = SciNdims;
      H.dims = dims;
      H.size = ntot;
      H.it = 0;
      H.type = 1;
      CreateHMat(num,&H);
      for (i=0;i<SciNdims;i++)  H.dims[i]=dims[i];
      *data=(void *)H.R;
    }
    *element_size=4;
    break;
  case CDF_INT4:
    /* 4 byte signed integer - should be okay on any platform */
    if (SciNdims==2) {
      l = I_INT32; CreateVar(num, "I", dims, dims+1, &l);
      *data=(void *)istk(l);
    }
    else {
      H.dimsize = SciNdims;
      H.dims = dims;
      H.size = ntot;
      H.it = I_INT32;
      H.type = 8;
      CreateHMat(num,&H);
      for (i=0;i<SciNdims;i++)  H.dims[i]=dims[i];
      *data=(void *)H.P;
    }
    *element_size=4;
    break;

  case CDF_UINT4:
    /* 4 byte unsigned integer - should be okay on any platform */
    if (SciNdims==2) {
      l = I_UINT32; CreateVar(num, "I", dims, dims+1, &l);
      *data=(void *)istk(l);
    }
    else {
      H.dimsize = SciNdims;
      H.dims = dims;
      H.size = ntot;
      H.it = I_UINT32;
      H.type = 8;
      CreateHMat(num,&H);
      for (i=0;i<SciNdims;i++)  H.dims[i]=dims[i];
      *data=(void *)H.P;
    }
    *element_size=4;
    break;

  case CDF_INT2:
    /* 2 byte signed integer - should be okay on any platform */
    if (SciNdims==2) {
      l = I_INT16; CreateVar(num, "I", dims, dims+1, &l);
      *data=(void *)istk(l);
    }
    else {
      H.dimsize = SciNdims;
      H.dims = dims;
      H.size = ntot;
      H.it = I_INT16;
      H.type = 8;
      CreateHMat(num,&H);
      for (i=0;i<SciNdims;i++)  H.dims[i]=dims[i];
      *data=(void *)H.P;
    }
    *element_size=2;
    break;

  case CDF_UINT2:
    /* 2 byte unsigned integer - should be okay on any platform */
    if (SciNdims==2) {
      l = I_UINT16; CreateVar(num, "I", dims, dims+1, &l);
      *data=(void *)istk(l);
    }
    else {
      H.dimsize = SciNdims;
      H.dims = dims;
      H.size = ntot;
      H.it = I_UINT16;
      H.type = 8;
      CreateHMat(num,&H);
      for (i=0;i<SciNdims;i++)  H.dims[i]=dims[i];
      *data=(void *)H.P;
    }
    *element_size=2;
    break;

  case CDF_INT1: case CDF_BYTE:
    /* 1 byte signed integer - should be okay on any platform */
    if (SciNdims==2) {
      l = I_INT8; CreateVar(num, "I", dims, dims+1, &l);
      *data=(void *)istk(l);
    }
    else {
      H.dimsize = SciNdims;
      H.dims = dims;
      H.size = ntot;
      H.it = I_INT8;
      H.type = 8;
      CreateHMat(num,&H);
      for (i=0;i<SciNdims;i++)  H.dims[i]=dims[i];
      *data=(void *)H.P;
    }
    *element_size=1;
    break;

  case CDF_UINT1:
    /* 1 byte unsigned integer - should be okay on any platform */
    if (SciNdims==2) {
      l = I_UINT8; CreateVar(num, "I", dims, dims+1, &l);
      *data=(void *)istk(l);
    }
    else {
      H.dimsize = SciNdims;
      H.dims = dims;
      H.size = ntot;
      H.it = I_UINT8;
      H.type = 8;
      CreateHMat(num,&H);
      for (i=0;i<SciNdims;i++)  H.dims[i]=dims[i];
      *data=(void *)H.P;
    }
    *element_size=1;
    break;

  case CDF_CHAR:case CDF_UCHAR:
    n1=ntot/dims[SciNdims-1];
    CreateStringVector(num,&n1,&(dims[SciNdims-1]),&l) ;
    *data=(void *)istk(l);
    *element_size=1;
    break;
  default:
    Scierror(999,"cdfread: Unknown CDF data type");
    return 0;
  }
  return 1;
}



/* for cre_hmat */
#define SCI_REAL_OR_CMPLX 1
#define SCI_POLYNOMIAL 2
#define SCI_IMPLICIT_POLY 129
#define SCI_BOOLEAN 4
#define SCI_SP_BOOLEAN 6
#define SCI_INTEGER 8
#define SCI_STRING 10
#define SCI_MLIST 17
#define SCI_INT8 1
#define SCI_UINT8 11
#define SCI_INT16 2
#define SCI_UINT16 12
#define SCI_INT32 4
#define SCI_UINT32 14
#define NOT_REAL_or_CMPLX_or_BOOL_or_INT -1
static int cre_hmat(int pos, HyperMat *H)
{
  /*  dans cette version, seuls les champs dimsize, size et it sont definis
   *  et on alloue alors la memoire des champs dims, R (et I si it=1) dans
   *  la pile scilab (juste  la place occupee par la variable).
   */
  static char *Str[]= { "hm","dims","entries"}; int m1=1,n1=3;
  int mL=3,nL=1,lL, one=1, lr, lc, lar, lac;
  CreateVar(pos,"m", &mL, &nL, &lL);
  CreateListVarFromPtr(pos,1,"S", &m1, &n1, Str);
  lr = 4; lar = -1;
  CreateListVarFrom(pos,2,"I", &one, &H->dimsize, &lr, &lar);
  H->dims = istk(lr);

  lar = -1; lac = -1;

  switch (H->type)
    {
    case (SCI_REAL_OR_CMPLX):
      CreateListCVarFrom(pos,3,"d", &H->it, &H->size, &one , &lr, &lc, &lar, &lac);
      H->R = stk(lr);
      if ( H->it == 1)
	H->I = stk(lc);
      return 1;
      
    case (SCI_BOOLEAN):
      CreateListVarFrom(pos, 3, "b", &H->size, &one, &lr, &lar);
      H->P = (char *) istk(lr);
      return 1;

    case (SCI_INTEGER):
      lr = H->it;
      CreateListVarFrom(pos, 3, "I", &H->size, &one, &lr, &lar);
      H->P = (char *) istk(lr);
      return 1;
    }

  /* Ajout Allan CORNET Correction Warning */
  /* warning C4715: 'cre_hmat' : not all control paths return a value */
  return 1;
	

}

