/*
 * Reference:
 * CDF C Reference Manual, version 3.1 January 2006 Space Physics Data Facility NASA 
 * Author 
 * S. Steer , Copyright INRIA, Marsh 2006
 * inspired of cdf2skt.c, 
 */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "cdf.h"

#include "stack-c.h"

/*
 * Calling format: data=cdf2date(times)
 *
 * where times is a n vector of CDF Epoch format times
 *       data  is a n by 6 matrix holding the converted values
 */
int SciCDFConvertEpoch(char * fname)
{

  int m1,n1,l1,l,N,i,li;
  long year;
  long month;
  long day;
  long hour;
  long minute;
  long second;
  long msec;

  /* get the times argument */
  GetRhsVar(1,"d",&m1,&n1,&l1);
  N=m1*n1;
  n1=6;
  CreateVar(2,"d",&N, &n1,&l);
  li=l;
    for (i=0; i<N; i++) {
      /* Convert to full format using CDF routine */
      EPOCHbreakdown(*stk(l1+i),&year,&month,&day,&hour,&minute,&second,&msec);
      /* Fill in the temporary array with these points */
      *stk(li)=(double)year;
      *stk(li+N)=(double)month;
      *stk(li+2*N)=(double)day;
      *stk(li+3*N)=(double)hour;
      *stk(li+4*N)=(double)minute;
      *stk(li+5*N)=(double)second+((double)msec/1000.0);
      li++;
    }
  LhsVar(1) = 2;
  return 0;
}
