/*
 * Reference:
 * CDF C Reference Manual, version 3.1 January 2006 Space Physics Data Facility NASA 
 * Author 
 * S. Steer , Copyright INRIA, Marsh 2006
 * inspired of cdf2skt.c
 */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "cdf.h"

#include "stack-c.h"

/*
 * returns a struct containing the CDF file global Attributes
 */


/* Function prototypes */
extern int cdfstatus(CDFstatus);
extern int CreateCDFVar(int num, long data_type,int ndims,int *dims,void **data,int *element_size);

#define CDFStatus(CDFstatus) if(!cdfstatus(CDFstatus))  {goto Lend;  }


int SciCDFGetGlobalAttributes(char* fname)
{
  int one=1;
  int n1,m1,l1;
  long i; /* Loop counters */

  CDFid id;             /* File ID */
  long attrN, numAttrs, scope, maxEntry;
  long dataType, numElements, entryN;
  char attrName[CDF_ATTR_NAME_LEN256+1];
  char **Str;
  int entryCount,strCount,GattrCount = 0;
  int Position,pos;
  int SciNdims;                 /* Number of dimensions of Scilab array to create */
  int SciDims[CDF_MAX_DIMS];    /* dimensions  of each Scilab variable - may be # as the CDF variable */
  void *SciArrayPtr;            /* The beginning of real data in a Scilab array */
  int CDFElementSize;           /* How many bytes an entry takes in CDF entry*/
  CDFstatus status;
 

  /* Check number of arguments */
  CheckRhs(1,1) ;
  CheckLhs(1,1) ;

  /* First argument should be a  string */
  GetRhsVar(1,"c",&m1,&n1,&l1);

  /*---------------------------------------------------------------*
   *-----  Opening the CDF file  ----------------------------------*
   *---------------------------------------------------------------*/
  if (CDFlib(OPEN_,CDF_,cstk(l1),&id,NULL_)!=CDF_OK) {
    /* That didn't work. Close it */
    CDFlib(CLOSE_,CDF_, NULL_);
    Scierror(999,"%s: File %s cannot be opened \r\n",fname,cstk(l1));
    return 0;
  }

  /* Make sure we're only reading the CDF file */
  CDFStatus(CDFlib(SELECT_,CDF_READONLY_MODE_,READONLYon, NULL_));

  /*---------------------------------------------------------------*
   *----- Count global scope attributes    ------------------------*
   *---------------------------------------------------------------*/
  

  CDFStatus(CDFlib (GET_,CDF_NUMATTRS_,&numAttrs,NULL_));
  if (numAttrs > 0) {
    /* count global scope attributes */
    GattrCount=0;
    for (attrN = 0; attrN < numAttrs; attrN++) {
      CDFStatus(CDFlib (SELECT_,ATTR_,attrN,GET_,ATTR_SCOPE_,&scope,NULL_));
      if (scope == GLOBAL_SCOPE) {GattrCount++;}
    }
  }
  Position=Rhs+1;
  
  pos=Position;
  if (GattrCount==0) { /* No global scope attributes return an empty matrix */
    n1=0;
    CreateVar(pos,"d",  &n1, &n1, &l1);
    pos++;
  }
  else {
    /*---------------------------------------------------------------*
     * Create the struct first field with the global attribute names *
     *---------------------------------------------------------------*/
    if (( Str= (char **) MALLOC((GattrCount+2)*sizeof(char*))) == NULL) {
      Scierror(999,"%s: running out of memory \r\n",fname);
      goto Lend;
    }
    Str[0]="st";
    Str[1]="dims";

    strCount=2;
    for (attrN = 0; attrN < numAttrs; attrN++) {
      CDFStatus(CDFlib (SELECT_,ATTR_,attrN,GET_,ATTR_NAME_,attrName,ATTR_SCOPE_,&scope,NULL_));
      if (scope == GLOBAL_SCOPE) { /* it is a global scope attribute*/
	if ((Str[strCount]= MALLOC((strlen(attrName)+1)*sizeof(char)))== NULL) {
	  Scierror(999,"%s: running out of memory \r\n",fname);
	  goto Lend1;
	}
	for (i = 0;i <  strlen(attrName); i++) {
	  if (attrName[i]==' ') {attrName[i]='_';}
	}
	strcpy(Str[strCount],attrName);
	strCount++;
      }
    }
    CreateVarFromPtr(pos, "S", &strCount, &one, Str);
    pos++;
    for (i = 0;i < strCount ; i++) {Str[i]=NULL;}
    FREE(Str);
    
    /*---------------------------------------------------------------*
     *------------ Create the struct dims field ---------------------*
     *---------------------------------------------------------------*/
    n1 = 2; 
    l1 = 4; CreateVar(pos, "I", &one, &n1, &l1);
    //    *istk(l1)=strCount-2;
    *istk(l1)=GattrCount;
    *istk(l1+1)=1;
    pos++;

    /*---------------------------------------------------------------*
     *------------ Create  an entry for each global attribute--------*
     *---------------------------------------------------------------*/
    GattrCount=0;
    for (attrN = 0; attrN < numAttrs; attrN++) {
      CDFStatus(CDFlib (SELECT_,ATTR_,attrN,GET_,ATTR_NAME_,attrName,ATTR_SCOPE_,&scope,NULL_));
      if (scope == GLOBAL_SCOPE) { /* it is a global scope attribute*/
	GattrCount++;
	CDFStatus(CDFlib(GET_,ATTR_MAXgENTRY_,&maxEntry,NULL_));

	/* here we do not know the number of char type entries so we allocate a 
	   string array of size maxEntry */
	if (( Str= (char **) MALLOC((maxEntry+1)*sizeof(char*))) == NULL) {
	  Scierror(999,"%s: running out of memory \r\n",fname);
	  goto Lend;
	}
	for (i = 0;i <  maxEntry; i++) {Str[i] = NULL;}

	strCount=0;
	entryCount=0;
	for (entryN = 0; entryN <= maxEntry; entryN++) {
	  status=CDFlib(SELECT_,gENTRY_,entryN,GET_,gENTRY_DATATYPE_,&dataType,
			gENTRY_NUMELEMS_, &numElements, NULL_);
	  if (status != NO_SUCH_ENTRY) {
	    CDFStatus(status);
	    if ((dataType==CDF_CHAR)||(dataType==CDF_UCHAR)) {
	      if ((Str[strCount]= MALLOC((numElements+1)*sizeof(char)))== NULL) {
		Scierror(999,"%s: running out of memory \r\n",fname);
		goto Lend1;
	      }
	      CDFStatus(CDFlib (GET_,gENTRY_DATA_,Str[strCount],NULL_));
	      Str[strCount][numElements]='\0';
	      strCount++;
	    }
	    else {
	      if (strCount>0) { 
		/* catenate consecutive CDF_CHAR entries in a scilab string array*/
		/* translate Str to a Scilab string array */
		CreateVarFromPtr(pos+entryCount, "S", &strCount, &one, Str);
		for (i=0;i<strCount;i++) {FREE(Str[i]);Str[i]=NULL;} 
		strCount=0;
		entryCount++;
	      }
	      SciNdims=2;
	      SciDims[0]=1;
	      SciDims[1]=(int)numElements;

	      if (dataType==CDF_REAL4||dataType==CDF_FLOAT) {
		float *f;
		double *d;
		if (!CreateCDFVar(pos+entryCount,dataType,SciNdims,SciDims,&SciArrayPtr,&CDFElementSize)) goto Lend1;
		if((f=MALLOC(numElements*sizeof(float)))==NULL) goto Lend1;
		d=(double *)SciArrayPtr;
		CDFStatus(CDFlib(GET_,zENTRY_DATA_,f,NULL_)); /*not ok with floats */
		for (i=0;i<numElements;i++) {d[i]=f[i];}
		FREE(f);
	      }
	      else {
		if (!CreateCDFVar(pos+entryCount,dataType,SciNdims,SciDims,&SciArrayPtr,&CDFElementSize)) goto Lend1;
		CDFStatus(CDFlib (GET_,gENTRY_DATA_,SciArrayPtr,NULL_)); 
	      }
	      entryCount++; 
	    }
	  }
	}
	if (strCount>0) { 
	  /* catenate consecutive CDF_CHAR entries in a scilab string array*/
	  /* translate Str to a Scilab string array */
	  CreateVarFromPtr(pos+entryCount, "S", &strCount, &one, Str);
	  for (i = 0; i < strCount; i++) {FREE(Str[i]);} 
	  strCount=0;
	  entryCount++;
	}
	/* end of current global attribute */
	if (entryCount>1) {
	  /*form a single list with all the entry of the current attribute */
	  C2F(mklistfromvars)(&pos, &entryCount); 
	}
	if (entryCount==0) {
	  /*form an empty matrix */
	  n1=0;
	  CreateVar(pos,"d",  &n1, &n1, &l1);
	}
	pos++;
      } /* if (scope == GLOBAL_SCOPE) */
    } /* for (attrN = 0; attrN < numAttrs; attrN++) */
    /*---------------------------------------------------------------*
     *------ Aggregate fields in a typer list to form the struct ----*
     *---------------------------------------------------------------*/
    GattrCount=GattrCount+2;
    C2F(mktlistfromvars)(&Position, &GattrCount);
  } /* global attributes */
  LhsVar(1) = Position;
  return 0;

 Lend1:
  if (strCount>0) { 
    for (i = 0; i < strCount; i++) {FREE(Str[i]);} 
    FREE(Str);
  }
 Lend:
  return 1;
}    
