/*
 * Reference:
 * CDF C Reference Manual, version 3.1 January 2006 Space Physics Data Facility NASA 
 * Author 
 * S. Steer , Copyright INRIA, Marsh 2006
 * inspired of cdf2skt.c
 */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "cdf.h"

#include "stack-c.h"

/*
 * returns a struct containing the CDF file variables Attributes
 */


/* Function prototypes */
extern int cdfstatus(CDFstatus);
extern int CreateCDFVar(int num, long data_type,int ndims,int *dims,void **data,int *element_size);
extern int createstringvect(integer *lw, integer *m, integer *nchar, integer *lr);
extern int C2F(cvstr)(integer * n,integer * line,char * str,integer * job,unsigned long str_len);
int Lnblnk(char *str);


#define CDFStatus(CDFstatus) if(!cdfstatus(CDFstatus))  {goto Lend;  }
#define CreateStringVector(n,m,nchar,lr) if(!createstringvect((c_local=n,&c_local),m,nchar,lr)) { return 0;  }
#define BOO(b,t,f) (b ? t : f)

int SciCDFGetVariableAttributes(char* fname)
{
  int job=0;
  int one=1;
  int n1,m1,l1,ld;
  long i; /* Loop counters */

  static char *Cell[]= { "ce","dims","entries"};
  CDFid id;             /* File ID */
  long attrN, numAttrs, scope;
  long dataType, numElements;
  long numZvars;
  long varN;
  char attrName[CDF_ATTR_NAME_LEN256+1];
  char VarName[CDF_VAR_NAME_LEN+1];
  char *Temp;
  int VattrCount = 0;
  int Position,pos,pos1;
  int SciNdims;                 /* Number of dimensions of Scilab array to create */
  int SciDims[CDF_MAX_DIMS];    /* dimensions  of each Scilab variable - may be # as the CDF variable */
  void *SciArrayPtr;            /* The beginning of real data in a Scilab array */
  int CDFElementSize;           /* How many bytes an entry takes in CDF entry*/
  CDFstatus status;
 
  /* Check number of arguments */
  CheckRhs(1,1) ;
  CheckLhs(1,1) ;

  /* First argument should be a  string */
  GetRhsVar(1,"c",&m1,&n1,&l1);

  /*---------------------------------------------------------------*
   *-----  Opening the CDF file  ----------------------------------*
   *---------------------------------------------------------------*/
  if (CDFlib(OPEN_,CDF_,cstk(l1),&id,NULL_)!=CDF_OK) {
    /* That didn't work. Close it */
    CDFlib(CLOSE_,CDF_, NULL_);
    Scierror(999,"%s: File %s cannot be opened \r\n",fname,cstk(l1));
    return 0;
  }

  /* Make sure we're only reading the CDF file */
  CDFStatus(CDFlib(SELECT_,CDF_READONLY_MODE_,READONLYon, NULL_));

  /* Select zMode2 */
  CDFStatus(CDFlib(SELECT_,CDF_zMODE_,zMODEon2,NULL_));


  /*---------------------------------------------------------------*
   *-------------- Count Variable attributes ----------------------*
   *---------------------------------------------------------------*/

  CDFStatus(CDFlib (GET_,CDF_NUMATTRS_,&numAttrs,NULL_));

  CDFStatus(CDFlib (GET_,CDF_NUMzVARS_,&numZvars,NULL_));

  VattrCount=0;
  if (numAttrs > 0) {
    /* count "variable attributes" */
    for (attrN = 0; attrN < numAttrs; attrN++) {
      CDFStatus(CDFlib (SELECT_,ATTR_,attrN,GET_,ATTR_SCOPE_,&scope,NULL_));
      if (scope == VARIABLE_SCOPE) {VattrCount++;}
    }
  }

  Position=Rhs+1;
  pos=Position;
  if (VattrCount==0) {/** return an empty matrix */
    n1=0;
    CreateVar(pos,"d",  &n1, &n1, &l1);
    pos++;
  }
  else {
    /*--------------------------------------------------------------------*
     *---------------------- create the cell first field -----------------*
     *--------------------------------------------------------------------*/
    n1 = 3;
    CreateVarFromPtr(pos, "S", &one, &n1, Cell);
    pos++;
    /* ----- create the dims field --------------*/
    n1 = 2; 
    ld = 4; CreateVar(pos, "I", &one, &n1, &ld);
    *istk(ld)= VattrCount+1;
    *istk(ld+1)=numZvars+1;
    pos++; 

    pos1=pos;
    /* top left entry of the cell */
    n1=20;    
    CreateStringVector(pos,&one,&n1,&l1) ;
    C2F(cvstr)(&n1,istk(l1),"Attributes/Variables",&job,n1);
    pos++;

    /* first column: the attributes name */
    for (attrN = 0; attrN < numAttrs; attrN++) {
      CDFStatus(CDFlib (SELECT_,ATTR_,attrN,GET_,ATTR_NAME_,attrName,ATTR_SCOPE_,&scope,NULL_));
      if (scope == VARIABLE_SCOPE) { /* it is a "variable  attribute"*/
	for (i = 0;i <  strlen(attrName); i++) {
	  if (attrName[i]==' ') {attrName[i]='_';}
	}
	n1=strlen(attrName);
	CreateStringVector(pos,&one,&n1,&l1) ;
	C2F(cvstr)(&n1,istk(l1),attrName,&job,n1);
	pos++;
      }
    }

    /*--------------------------------------------------------------------*
     *----------  create a column for each variable ---------------- -----*
     *--------------------------------------------------------------------*/


    /* ----- now loop to store the variable names and the value of the attributes */
    for (varN = 0; varN < numZvars; varN++) {
      CDFStatus(CDFlib(SELECT_,zVAR_,(long)varN,NULL_));

      /*store the variable names */
      CDFStatus(CDFlib(GET_,zVAR_NAME_,VarName,NULL_));
      n1=Lnblnk(VarName);
      CreateStringVector(pos,&one,&n1,&l1) ;
      C2F(cvstr)(&n1,istk(l1),VarName,&job,n1);
      pos++;

      for (attrN = 0; attrN < numAttrs; attrN++) {
	CDFStatus(CDFlib (SELECT_,ATTR_,attrN,GET_,ATTR_NAME_,attrName,ATTR_SCOPE_,&scope,NULL_));
	if (scope == VARIABLE_SCOPE) { /* it is a "variable  attribute"*/
	  status = CDFlib (SELECT_, BOO(1,zENTRY_,rENTRY_), varN,
			   GET_, BOO(1,zENTRY_DATATYPE_,
				     rENTRY_DATATYPE_), &dataType,
			   BOO(1,zENTRY_NUMELEMS_,
			       rENTRY_NUMELEMS_), &numElements,
			   NULL_);
	  if (status != NO_SUCH_ENTRY) {
	    CDFStatus(status);
	    /*STORE THE ATTRIBUTE VALUE */
	    SciNdims=2;
	    SciDims[0]=1;

	    if (dataType==CDF_CHAR) {
	      if ((Temp= MALLOC((numElements+1)*sizeof(char)))== NULL) {
		Scierror(999,"%s: running out of memory \r\n",fname);
		goto Lend;
	      }
	      CDFStatus(CDFlib(GET_,zENTRY_DATA_,Temp,NULL_)); /*not ok with floats */
	      Temp[numElements]='\0';
	      n1=Lnblnk(Temp);
	      SciDims[1]=n1;
	      if (!CreateCDFVar(pos,dataType,SciNdims,SciDims,&SciArrayPtr,&CDFElementSize)) goto Lend;
	      
	      C2F(cvstr)(&n1,(int *)SciArrayPtr,Temp,&job,n1);
	      FREE(Temp);
	    }
	    else if (dataType==CDF_REAL4||dataType==CDF_FLOAT) {
	      float *f;
	      double *d;
	      if((f=MALLOC(numElements*sizeof(float)))==NULL) goto Lend;
	      
	      SciDims[1]=(int)numElements;
	      if (!CreateCDFVar(pos,dataType,SciNdims,SciDims,&SciArrayPtr,&CDFElementSize)) goto Lend;
	      d=(double *)SciArrayPtr;
	      CDFStatus(CDFlib(GET_,zENTRY_DATA_,f,NULL_)); /*not ok with floats */
	      for (i=0;i<numElements;i++) {d[i]=f[i];}
	      FREE(f);
	    }
	    else {
	      SciDims[1]=(int)numElements;
	      if (!CreateCDFVar(pos,dataType,SciNdims,SciDims,&SciArrayPtr,&CDFElementSize)) goto Lend;
	      CDFStatus(CDFlib(GET_,zENTRY_DATA_,SciArrayPtr,NULL_)); /*not ok with floats */
	    }
	    pos++;
	  }
	  else {
	    /* ----- no variable attribute , store an empty matrix ---*/
	    n1=0;
	    CreateVar(pos,"d",  &n1, &n1, &l1);
	    pos++;
	  }
	} /*scope== VARIABLE-SCOPE*/
      }/* for (attrN = 0; attrN < numAttrs; attrN++) */
    } /*for (varN = 0; varN < numZvars; varN++)*/
    n1 = (VattrCount+1)*(numZvars+1);
    C2F(mklistfromvars)(&pos1, &n1); /*form the cell value list */ 
    pos1=pos1-2;n1=3; /* form the cell */
    C2F(mktlistfromvars)(&pos1, &n1); 
    LhsVar(1) = Rhs+1;
  }
  return 0;
  
 Lend:
  return 1;
}    

int Lnblnk(char *str)
{
  int i;
  for (i=strlen(str)-1;i>0;i--) {
    if (str[i]!=' ') break;
  }
  return i+1;
}

