#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "cdf.h"

#include "stack-c.h"
/*
 * Calling format: cdfload(filename [,varnames])
 *
 * where filename is the name of the CDF file to open
 *       varnames is an array of variable names from the CDF file which
 *                   will be returned
 *       extract the requested variables and store them with their CDF names
 *
 * Reference:
 * CDF C Reference Manual, version 3.1 January 2006 Space Physics Data Facility NASA 
 * Author 
 * S. Steer , Copyright INRIA, Marsh 2006
 */

/* Function prototypes */
extern int cdfstatus(CDFstatus);
extern int SciCDFReadSelectedVariable(int num,long CDFmajority );
extern void C2F(stackp)(int *Scivarname, int *flag);
#define CDFStatus(CDFstatus) if(!cdfstatus(CDFstatus))  {goto Lend;  }

int SciCDFLoad(char * fname)
{
  /* constants*/
  int job=0;
  int flag=1;

  /* General variables */
  int i,j,m1,n1,l1;     
  int Scivarname[6];

  CDFid Fid;                     /* File ID */
  char **Names=NULL;            /* string array used to get the second argument value */
  long CDFmajority;             /* Majority of current CDF file */
  long numvars;

  /* Check number of arguments */
  CheckRhs(1,2) ;

  /* First argument (file) should be a  string */
  GetRhsVar(1,"c",&m1,&n1,&l1);
  CheckLhs(1,1) ;

  /* Open the CDF file */
  if (CDFlib(OPEN_,CDF_,cstk(l1),&Fid,NULL_)!=CDF_OK) {
    /* That didn't work. Close it */
    CDFlib(CLOSE_,CDF_, NULL_);
    Scierror(999,"%s: File %s cannot be opened \r\n",fname,cstk(l1));
    goto Lend;
  }

  /* Make sure we're only reading the CDF file */
  CDFStatus(CDFlib(SELECT_,CDF_READONLY_MODE_,READONLYon,NULL_));

  /* Select zMode2 */
  CDFStatus(CDFlib(SELECT_,CDF_zMODE_,zMODEon2,NULL_));

  /* Find majority encoding - this is a file-wide attribute */
  CDFStatus(CDFlib(GET_,CDF_MAJORITY_,&CDFmajority,NULL_));

 /* How many z entries are there? */
  CDFStatus(CDFlib(GET_,CDF_NUMzVARS_,&numvars,NULL_));

  if (Rhs==2) {
  /* Second argument (names) should be a vector of strings */
    GetRhsVar(2,"S",&m1,&n1,&Names);
    numvars=m1*n1;
  }
  else {
    /* Allocate array for variable names */
    if (( Names = (char **) MALLOC((int)numvars*sizeof(char*))) == NULL) {
      Scierror(999,"%s: running out of memory \r\n",fname);
      goto Lend;
    }
    for (i=0;i<numvars;i++)
      {
	if ((Names[i]=(char *)  MALLOC((CDF_VAR_NAME_LEN+1)*sizeof(char))) == NULL) {
	  for (j=0;j<i;j++) {FREE(Names[j]);}
	  FREE(Names);
	  Scierror(999,"%s: running out of memory \r\n",fname);
	  goto Lend;
	}
	
      }
  }
  /* Loop through requested variables*/
  for (i=0;i<(int)numvars;i++) {
    if (Rhs==1) {
      /* Select the variable. */
      CDFStatus(CDFlib(SELECT_,zVAR_,(long)i,NULL_));
      /* Get the name of the variable */
      CDFStatus(CDFlib(GET_,zVAR_NAME_,Names[i],NULL_));
    }
    else {
      CDFStatus(CDFlib(SELECT_,zVAR_NAME_,Names[i],NULL_));
    }
    if (SciCDFReadSelectedVariable(Rhs+1,CDFmajority)) goto Lend1;

    /* store the variable in the stack under the name Names[i]*/
    C2F(cvname)(Scivarname,Names[i],&job,strlen(Names[i]));
    Top=Top+1; 
    C2F(stackp)(Scivarname,&flag);
    Nbvars--;
  }
  /* Create the output variable */
  n1=numvars;
  CreateVarFromPtr(Rhs+1, "S", &n1, &flag, Names);

 Lend1:
  /* Free the Names string array */
  for (j=0;j<i;j++) {FREE(Names[j]);}
  FREE(Names);
 Lend:
  /* Close the CDF file */
  CDFlib(CLOSE_,CDF_,NULL_);
  LhsVar(1)=Rhs+1;
  return  0;
}

