#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "cdf.h"

#include "stack-c.h"
/*
 * Calling format: [V1,V2,..,Vn]=cdfread(filename,varnames)
 *
 * where filename is the name of the CDF file to open
 *       varnames is an array of variable names from the CDF file which
 *                   will be returned
 *       Vi     The returned Scilab variables
 *
 * Reference:
 * CDF C Reference Manual, version 3.1 January 2006 Space Physics Data Facility NASA 
 * Author 
 * S. Steer , Copyright INRIA, Marsh 2006
 * inspired of cdfread.c, Tim Horbury  1999, Astronomy Unit, Queen Mary and Westfield College, London
 */

extern int C2F(cvstr)(integer * n,integer * line,char * str,integer * job,unsigned long str_len);



/* Function prototypes */
extern int cdfstatus(CDFstatus);
extern int SciCDFReadSelectedVariable(int num,long CDFmajority );
#define CDFStatus(CDFstatus) if(!cdfstatus(CDFstatus))  {goto Lend;  }

int SciCDFRead(char * fname)
{
  /* General variables */
  integer m1,n1,l1;
  long i,N;     

  CDFid Fid;                     /* File ID */
  char **Names=NULL;            /* string array used to get the second argument value */
  long CDFmajority;             /* Majority of current CDF file */

  /* Check number of arguments */
  CheckRhs(2,2) ;

  /* First argument (file) should be a  string */
  GetRhsVar(1,"c",&m1,&n1,&l1);
  /* Second argument (names) should be a vector of strings */
  GetRhsVar(2,"S",&m1,&n1,&Names);
  N=m1*n1;
  /* Check there is as many lhs arguments as names */
  CheckLhs(N,N) ;


  /* Open the CDF file */
  if (CDFlib(OPEN_,CDF_,cstk(l1),&Fid,NULL_)!=CDF_OK) {
    /* That didn't work. Close it */
    CDFlib(CLOSE_,CDF_, NULL_);
    Scierror(999,"%s: File %s cannot be opened \r\n",fname,cstk(l1));
    goto Lend;
  }

  /* Make sure we're only reading the CDF file */
  CDFStatus(CDFlib(SELECT_,CDF_READONLY_MODE_,READONLYon,NULL_));

  /* Select zMode2 */
  CDFStatus(CDFlib(SELECT_,CDF_zMODE_,zMODEon2,NULL_));

  /* Find majority encoding - this is a file-wide attribute */
  CDFStatus(CDFlib(GET_,CDF_MAJORITY_,&CDFmajority,NULL_));

  /* Loop through requested variables*/
  for (i=0;i<N;i++) {
    CDFStatus(CDFlib(SELECT_,zVAR_NAME_,Names[i],NULL_));
    if (SciCDFReadSelectedVariable(Rhs+1+i,CDFmajority)) goto Lend;
    LhsVar(i+1) = Rhs+1+i;
  }
 Lend:
  FreeRhsSVar(Names);
  /* Close the CDF file */
  CDFlib(CLOSE_,CDF_,NULL_);
  return  0;
}






