#include <string.h>
#include "cdf.h"
#include "stack-c.h"
/*
 * Reference:
 * CDF C Reference Manual, version 3.1 January 2006 Space Physics Data Facility NASA 
 * Author 
 * S. Steer , Copyright INRIA, Marsh 2006
 * inspired of cdf2skt.c, 
 */

extern int cdfstatus(CDFstatus);
extern int CDF_Index(int *x,int *dims,int Ndims,long majority);
extern int Scilab_Index(int *x,int *dims,int Ndims,long majority);
extern int CreateCDFVar(int num, long data_type,int ndims,int *dims,void **data,int *element_size);extern int C2F(cvstr)(int * n,int * line,char * str,int * job,unsigned long str_len);

#define CDFStatus(CDFstatus) if(!cdfstatus(CDFstatus))  {goto Lend;  }

int  SciCDFReadSelectedVariable(int pos,long CDFmajority )
{

  /* constants*/
  int job=0;
  int DoubleSize=8;

  /* General variables */
  long j,krec; 
  int n1;    
  float f;                      /* for float to double conversion */
  double d;                     /* for float to double conversion */
  int is_vector;                /* set to true if only one dim is >1 */
  int change_majority;          /* set to true if majority has to be changed */
  long num_entries;             /* Number of elements  */

  int  CDFElementSize;           /* How many bytes an entry takes in CDF entry*/
  int  SciElementSize;
  int  SciNdims;                 /* Number of dimensions of Scilab array to create */
  int  SciDims[CDF_MAX_DIMS+1];    /* dimensions  of each Scilab variable - may be # as the CDF variable */
  long recVary, dimVarys[CDF_MAX_DIMS];
  int  Indexes[CDF_MAX_DIMS];    /* Array of dimension index values */
  int  CDFIndex;                 /* index along temporary array when  copying to Scilab array */
  int  SciIndex;                 /* index along Scilab array when copying from temporary array */
  long numDims;                /* Number of dimensions in current CDF variable */
  long dimSizes[CDF_MAX_DIMS];   /* Size of each dimension in current CDF variable */
  long maxRec;               /* Number of records in current CDF variable */
  long dataType;             /* CDF type of current variable */
  long CDFDimPars[CDF_MAX_DIMS];/* Array to pass parameters to CDFlib for hyperreads */
  long CDFStringLength;         /* Number of elements of the data type at each value. Only for CDF_CHAR */
  void *TmpArrayPtr;            /* Pointer to data storage array for current variable */
  void *SciArrayPtr;            /* The beginning of real data in a Scilab array */



  CDFStatus(CDFlib(GET_,zVAR_DATATYPE_,&dataType,zVAR_MAXREC_,&maxRec,
		   zVAR_NUMELEMS_,&num_entries,zVAR_RECVARY_,&recVary,
		   zVAR_DIMVARYS_,dimVarys,NULL_));

   
  /* Number of dimensions of this variable*/
  CDFStatus(CDFlib(GET_,zVAR_NUMDIMS_,&numDims,zVAR_DIMSIZES_,dimSizes,NULL_));

 
  //  sciprint("numDims=%d :",numDims);
  //for (i=0;i<numDims;i++) sciprint("%d ",dimSizes[i]);
  //sciprint(",   nRecvalues=%d maxRec=%d\n",nRecValues,maxRec);

  /* Convert to Scilab dimension */
  if (numDims==1) { /* a vector ->row */
    SciNdims=2;
    SciDims[0]=(int)dimSizes[0];
    SciDims[1]=1;
  }
  else if (numDims==0) { /* a scalar */
    /* Scilab arrays have a minimum of two dimensions */
    SciNdims=2;
    SciDims[0]=1;
    SciDims[1]=1;
  }
  else { /* matrix or hyper matrix*/
    CDFStatus(CDFlib(GET_,zVAR_DIMSIZES_,dimSizes,NULL_));
    SciNdims=(int)numDims;
    for (j=0; j<numDims; j++) {SciDims[j]=(int)dimSizes[j];}
  }
  //  for (i=0;i<SciNdims;i++) sciprint("%d ",SciDims[i]);
  //sciprint(" Sci \n\n");

  /* Test if the Scilab array is a vector (majority does not matter)*/
  is_vector=0;
  for (j=0; j<SciNdims; j++) {if (SciDims[j]>1) is_vector++;}
  is_vector=is_vector==1;

  /* Special case: character arrays */
  if (dataType==CDF_CHAR) {
    /* How many character per string? */
    CDFStatus(CDFlib(GET_,zVAR_NUMELEMS_,&CDFStringLength,NULL_));
    SciNdims+=1;
    SciDims[SciNdims-1]=(int)CDFStringLength;
  }

  /* Calculate total number of entries in Scilab variable */
  num_entries=1;
  for (j=0; j<SciNdims; j++) {num_entries=num_entries*(long)SciDims[j];}

  /* Allocate the Scilab variable */
  if (maxRec>0) {
    if (SciDims[SciNdims-1]==1) {
      SciDims[SciNdims-1]=maxRec+1;
      if (!CreateCDFVar(pos,dataType,SciNdims,SciDims,&SciArrayPtr,&CDFElementSize)) goto Lend;
      SciDims[SciNdims-1]=1;
    }
    else {
      /* increase the number of dimension */
      SciDims[SciNdims]=(maxRec+1);
      if (!CreateCDFVar(pos,dataType,SciNdims+1,SciDims,&SciArrayPtr,&CDFElementSize)) goto Lend;
    }
  }
  else {  
    if (!CreateCDFVar(pos,dataType,SciNdims,SciDims,&SciArrayPtr,&CDFElementSize)) goto Lend;
  }

  SciElementSize=CDFElementSize;
  if(dataType==CDF_CHAR||dataType==CDF_UCHAR)
    { SciElementSize=4;}
  else if(dataType==CDF_REAL4) 
    { SciElementSize=8;}

  /* Define the output array for hyperread*/
  if ((!is_vector) && (CDFmajority!=ROW_MAJOR)) {
    /* Majority or type  has to be changed, define a temporary storage array*/
    if ((TmpArrayPtr=CALLOC(num_entries,CDFElementSize))==NULL) {goto Lend1;}
    change_majority=1; 
  }
  else if(dataType==CDF_CHAR||dataType==CDF_UCHAR||dataType==CDF_REAL4) {
    if ((TmpArrayPtr=CALLOC(num_entries,CDFElementSize))==NULL) {goto Lend1;}
    change_majority=0;
  }
  else {
    TmpArrayPtr=SciArrayPtr;
    change_majority=0; 
  }
  

  for (krec=0;krec<=maxRec;krec++) {

    /* Select parameters for a hyperread */
    /* RECCOUNT: number of records to read: read everything in one go */
    CDFStatus(CDFlib(SELECT_,zVAR_RECCOUNT_,1, NULL_));
    /* Start at the first (krec th) record */
    CDFStatus(CDFlib(SELECT_,zVAR_RECNUMBER_,krec,NULL_));
    /* RECINTERVAL: step value between record reads. Read every record */
    CDFStatus(CDFlib(SELECT_,zVAR_RECINTERVAL_,(long)1,NULL_));
    
    if (numDims>0) {
      /* Set parameters for different dimensions. Use CDFDimPars array */
      /* DIMINDICES: where to start hyperread for each dimension */
      /* Always the beginning of each dimension */
      for (j=0; j<numDims; j++) CDFDimPars[j]=(long)0;
      
      CDFStatus(CDFlib(SELECT_,zVAR_DIMINDICES_,CDFDimPars,NULL_));
      
      /* DIMCOUNTS: number of elements to read in each dimension */
      /* Always the size of each dimension */
      for (j=0; j<numDims; j++) CDFDimPars[j]=dimSizes[j];	
      CDFStatus(CDFlib(SELECT_,zVAR_DIMCOUNTS_,CDFDimPars,NULL_));

      /* DIMINTERVALS: step between element reads for each dimension */
      /* Always 1 for each dimension */
      for (j=0; j<numDims; j++) CDFDimPars[j]=(long)1;
      CDFStatus(CDFlib(SELECT_,zVAR_DIMINTERVALS_,CDFDimPars,NULL_));
    } /* End of  numDims>0 case */

      /* Finished setting up for hyperread */

      /* Do a hyperread of a record into temporary array in one go */
    CDFStatus(CDFlib(GET_,zVAR_HYPERDATA_,TmpArrayPtr,NULL_));
    
     

    /* perform requested transformation on read data */
    if (!change_majority) {
      double *D=(double *)SciArrayPtr;
      float *F=(float *)TmpArrayPtr;
      switch (dataType) {
      case CDF_REAL4: 
	for (j=0; j<num_entries; j++) D[j]=(double)F[j]; 
	break;
      case CDF_CHAR: case CDF_UCHAR: 
	n1=num_entries;
	C2F(cvstr)(&n1,(int *)SciArrayPtr,(char *)TmpArrayPtr,&job,num_entries);
	break;
      }
    }
    else {
      /* Resort the hyperread array into the order that Scilab needs (COLUMN_MAJOR) */
      
      /* Initialise to zero Array for referencing dimensions when reading data*/
      for (j=0; j<SciNdims; j++) Indexes[j]=0; 
      
      /* Keep going until we run off the end of the lowest dimension */
      while (Indexes[0]<SciDims[0]) {
	/* Convert the indices here to pointer indices for source */
	CDFIndex=CDF_Index(Indexes,SciDims,SciNdims,CDFmajority);
	/* Same for the target */
	SciIndex=Scilab_Index(Indexes,SciDims,SciNdims,ROW_MAJOR);
	if(dataType==CDF_REAL4){
	  memcpy((char *)(&f),(char *)TmpArrayPtr+CDFIndex*CDFElementSize,CDFElementSize );
	  d=(double)f;
	  memcpy((char *)SciArrayPtr+SciIndex*DoubleSize,(char *)(&d),DoubleSize);
	}
	else if(dataType==CDF_CHAR||dataType==CDF_UCHAR){
	  Scierror(999,"cdfread: Matrix of strings not yet handled");
	  if (TmpArrayPtr!=SciArrayPtr) FREE(TmpArrayPtr);
	  goto Lend1;
	}
	else {
	  /* We can now place the right number of bytes */
	  memcpy((char *)SciArrayPtr+SciIndex*CDFElementSize,
		 (char *)TmpArrayPtr+CDFIndex*CDFElementSize,CDFElementSize );
	}
	
	/* We loop fastest over the highest dimension */
	Indexes[SciNdims-1]+=1;
	/* Check that we haven't run off the end of any dimension: if we */
	/* have, loop onto the next line, unless we're at the end (when we */
	/* run off the first dimension */
	j=SciNdims-1;
	while (Indexes[j]>=SciDims[j]) {
	  if (j>0) {
	    Indexes[j-1]+=1;
	    Indexes[j]=0;
	  }
	  j-=1;
	}
      }   /* Finished re-sorting array */
    }
    if (TmpArrayPtr==SciArrayPtr) TmpArrayPtr=(void *)((char *)SciArrayPtr+SciElementSize*num_entries);
    SciArrayPtr=(void *)((char *)SciArrayPtr+SciElementSize*num_entries);
    
  }
  
  /* Free TmpArray if it has been allocated */
 Lend:
  if (TmpArrayPtr!=SciArrayPtr) FREE(TmpArrayPtr);
  return 0;
 Lend1:
  return 1;
    
}
  
